#define LINT_ARGS 1      /* Enable Compiler Parameter Checking */
#include <stdio.h>
#include <math.h>

/*

Calculate means within some bins of data.

The program expects a command-line parameter, which is the
number of configurations within which averaging is performed.

The input format is:

     1) A parameter header largely irrelevant to this program,
        except that two integers - 'num_grps' and 'grp_size', 
        defining the grouping of input data - are extracted
        from it.

     2) A series, terminated by EOF, of "configurations".  Each 
        configuration is a set of (num_grps * grp_size) floating-
        point numbers.

*/


/* Data sizes... */

#define MAX_MEANS 2000
#define MAX_GRPS  1000


/* Static workspace... */

double means_acc [MAX_MEANS] ;



void init_arrays (mean_grp, num_grps, grp_size)

double  *mean_grp [MAX_GRPS] ;
int num_grps, grp_size ;

/*

  Initialise pointer arrays so that 'mean_grp' looks like an array of 
  linear arrays.

  Also initialise data to zeros.

*/

{
  {
    int num_means = num_grps * grp_size ;

    if (num_grps > MAX_GRPS)
    {
      fprintf (stderr, "\nERROR: Too many groups of observables\n") ;
      exit (1) ;
    }
    if (num_means > MAX_MEANS)
    {
      fprintf (stderr, "\nERROR: Too many observables\n") ;
      exit (1) ;
    }
  }

  {
    /*

    Initialise pointer arrays...

    double  *mean_grp [MAX_GRPS] ;

    */

    int g, i ;
    double *mean_addr = means_acc ;

    for (g = 0; g < num_grps; g++)
    {
      mean_grp [g] = mean_addr ;
      for (i = 0; i < grp_size; i++)
      {
        mean_addr     ++ ;
      }
    }
  }

  {

    /* Finally, zero arrays */

    int g, i, j ;

    for (g = 0; g < num_grps; g++)
      for (i = 0; i < grp_size; i++)
      {
        mean_grp [g] [i] = 0.0 ;
      }
  }
}

int add_sample (mean_grp, num_grps, grp_size)

double  *mean_grp [MAX_GRPS] ;
int num_grps, grp_size ;

/*

   Get a configuration, and add its contribution to arrays.

*/

{
  int g, i, j ;

  for (g = 0; g < num_grps; g++)
  {
    float obs [MAX_MEANS] ;
    {
      /* Read in group data */

      for (i = 0; i < grp_size; i++)
      {
        int result ;

        result = scanf ("%f", obs + i) ;
        if (result == EOF)
          if (g | i)
          {
            fprintf (stderr, "\nERROR: Unexpected end of file\n") ;
            exit (1) ;
          }
          else
            return 0 ;
      }

      /* Add it to arrays */

      for (i = 0; i < grp_size; i++)
      {
        mean_grp [g] [i] += obs [i] ;
      }
    }
  }
  return 1 ;
}

void output_group (means, grp_size) 

double  *means ;
int grp_size ;

/*

  Pretty-print group of means.

*/

{
  if (grp_size == 1)
    printf ("%13e\n", means [0]) ;
  else
  {
    int i, j, k ;

    for (i = 0; i < grp_size; i++)
      printf ("%13e\n", means [i]) ;

    printf ("\n") ;
  }
}

void process_sums (mean_grp, num_grps, grp_size, num_samples) 

double  *mean_grp [MAX_GRPS] ;
int num_grps, grp_size, num_samples ;

/*

  Normalisations, and stuff.  Ouput results.

*/

{
  if (num_samples > 0)
  {
    int g, i, j ;

    for (g = 0; g < num_grps; g++)
    {

      /* Normalise means */

      for (i = 0; i < grp_size; i++)
        mean_grp [g] [i] /= num_samples ;

      /* Output results */

      output_group (mean_grp [g], grp_size) ;
    }
  }
  else
    fprintf (stderr, "\nERROR: No data!\n") ;
}

void main (argc, argv)
int argc ;
char *argv [] ;
{
  int num_grps, grp_size ;
  int bin_size ;
  double  *mean_grp [MAX_GRPS] ;

  if (argc != 2)
  {
    fprintf (stderr, "ERROR: expecting one argument\n") ;
    exit (1) ;
  }
  if ((bin_size = atoi (argv [1])) == 0)
  {
    fprintf (stderr, "ERROR: invalid argument\n") ;
    exit (1) ;
  }

  read_header (&num_grps, &grp_size) ;

  while (1)
  {
    int i ;

    init_arrays (mean_grp, num_grps, grp_size) ;

    for (i = 0; i < bin_size; i++)
      if (!add_sample (mean_grp, num_grps, grp_size))
      {
	if (i)
	  fprintf (stderr, "WARNING: EOF encounterd.  Bin aborted.\n") ;
	exit (0) ;
      }

    process_sums (mean_grp, num_grps, grp_size, bin_size) ;
  }
}

/*
--
--     End of File.
--
*/
